-- AddOn by Dezyne, Argent Dawn, Europe. Please give me credit when you want to redistribute or modify this addon! Contact: coenvdwel@planet.nl
													 local rewatch_version = 3.0; -- current revision
--// FUNCTIONS //----------------------------------------------------------------------------------------------------------------------------------------------------------------

-- display a message to the user in the chat pane
-- msg: the message to pass onto the user
-- return: void
function rewatch_Message(msg)
	-- send the message to the chat pane, in yellow-ish
	DEFAULT_CHAT_FRAME:AddMessage(rewatch_loc["prefix"]..msg, 0.95, 0.85, 0.15);
	
	-- return
	return;
end;

-- trigger a cooldown overlay
-- spellName: the name of the spell to trigger the overlay from
-- return: void
function rewatch_TriggerCooldown(spellName)
	-- enable global cooldown overlay
	if(not spellName) then
		local start, duration, enabled = GetSpellCooldown(rewatch_loc["rejuvenation"]); -- some non-cd spell
		CooldownFrame_SetTimer(rewatch_f_gcd, start, duration, enabled);
	-- only get cooldown of the spell if there's a button for it
	elseif(rewatch_abs[spellName]["Button"]) then
		-- get the cooldown on the spell
		local start, duration, enabled = GetSpellCooldown(spellName);
		-- check if the spell even exists (non-resto, or low leveled)
		if(start == nil) then
			-- hide the button
			rewatch_abs[spellName]["Button"]:Hide();
		else
			-- set the cooldown frame timer on the cooldown overlay of the spell's button
			CooldownFrame_SetTimer(rewatch_abs[spellName]["Button"].cooldown, start, duration, enabled);
		end;
	end;
	
	-- return
	return;
end;

-- gets the spell ID of the highest rank of the specified spell
-- spellName: the name of the spell to get the highest ranked spellId from
-- return: the corresponding spellId
function rewatch_GetSpellId(spellName)
	-- get spell info and highest rank, return if the user can't cast it (not learned, etc)
	local name, rank, icon = GetSpellInfo(spellName);
	if(name == nil) then return -1; end;
	-- loop through all book spells, return the number if it matches above data
	local i, ispell, irank = 1, GetSpellName(1, BOOKTYPE_SPELL);
	repeat
		-- return if the name and rank equal
		if ((ispell == name) and ((rank == irank) or (not irank))) then
			return i;
		end;
		-- initialise next iteration
		i, ispell, irank = i+1, GetSpellName(i+1, BOOKTYPE_SPELL);
	until (not ispell);
	
	-- return default -1
	return -1;
end;

-- adjusts the parent frame container's height
-- d: the change in height to be processed; negative for shrinking, positive for enlarging
-- return: void
function rewatch_AlterFrame(d)
	-- alter the height directly
	rewatch_f:SetHeight(rewatch_f:GetHeight() + d);
	rewatch_f_gcd:SetHeight(rewatch_f:GetHeight());
	
	-- return
	return;
end;

-- checks if the player is in the group
-- player: name of the player to check for
-- return: true, if the player is the user, or in the user's party or raid; false elsewise
function rewatch_InGroup(player)
	-- default return value to false
	local r = false;
	-- catch a self-check; return true if searching for the user itself
	if(UnitName("player") == player) then return true;
	-- process by environment (party or raid)
	elseif(GetNumRaidMembers() > 0) then if(UnitPlayerOrPetInRaid(player)) then r = true; end;
	elseif(GetNumPartyMembers() > 0) then if(UnitPlayerOrPetInParty(player)) then r = true; end;
	end;
	
	-- return
	return r;
end;

-- round a number to the supplied number of decimals
-- num: the number to round
-- dec: the number of decimals to show
-- return: floor(num * 10^dec + 0.5) / 10^dec
function rewatch_Round(num, dec)
	-- default the dec to 0 and create multiply factor
    local mult = 10 ^ (dec or 0);
	
	-- return
    return math.floor(num * mult + 0.5) / mult;
end;

-- get the number of the supplied player's place in the player table, or -1
-- player: name of the player to search for
-- return: the supplied player's table index, or -1 if not found
function rewatch_GetPlayer(player)
	-- for every seen player
	for i,val in ipairs(rewatch_bars) do
		-- return if the name matches the supplied name
		if(val["Player"] == player) then return i; end;
	end;
	
	-- return -1 if not found
	return -1;
end;

-- add a player to the players table and create his bars and button
-- player: the name of the player
-- return: the index number the player has been assigned
function rewatch_AddPlayer(player)
	-- return if in combat
	if(InCombatLockdown() == 1) then return -1; end;
	
	-- create heading
	rewatch_bars[rewatch_i] = {}; rewatch_AlterFrame(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]);
	local statusbar = CreateFrame("STATUSBAR", nil, rewatch_f, "TextStatusBar"); statusbar:SetWidth(rewatch_data["SpellBarWidth"]); statusbar:SetHeight(rewatch_data["SpellBarHeight"]-5); statusbar:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, (rewatch_data["SpellBarHeight"]*2)-(rewatch_i+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	statusbar:SetBackdrop({bgFile = nil, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = 1, tileSize = 5, edgeSize = 5, insets = { left = 0, right = 0, top = 0, bottom = 0 }});
	statusbar:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar"); statusbar:SetStatusBarColor(0.0, 0.6, 0.0, 0.8);
	statusbar:SetMinMaxValues(0, 1); statusbar:SetValue(0);
	-- put text in heading
	local statusbarText = statusbar:CreateFontString("$parentText", "ARTWORK", "GameFontHighlightSmall");
	statusbarText:SetPoint("LEFT", statusbar); statusbarText:SetAllPoints(); statusbarText:SetText(player);
	-- energy/mana/rage bar
	local statusbar2 = CreateFrame("STATUSBAR", nil, rewatch_f, "TextStatusBar"); statusbar2:SetWidth(rewatch_data["SpellBarWidth"]); statusbar2:SetHeight(5); statusbar2:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, rewatch_data["SpellBarHeight"]-(rewatch_i+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"])+5);
	statusbar2:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = 1, tileSize = 5, edgeSize = 5, insets = { left = 0, right = 0, top = 0, bottom = 0 }});
	statusbar2:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar"); statusbar2:SetStatusBarColor(0.0, 0.0, 0.8, 0.8);
	statusbar2:SetMinMaxValues(0, 1); statusbar2:SetValue(0);
	-- overlay target/remove button
	local tgb = CreateFrame("BUTTON", nil, statusbar, "SecureActionButtonTemplate");
	tgb:SetWidth(statusbar:GetWidth()); tgb:SetHeight(statusbar:GetHeight()); tgb:SetPoint("TOPLEFT", statusbar, "TOPLEFT", 0, 0);
	tgb:RegisterForClicks("LeftButtonDown"); tgb:SetAttribute("type1", "target"); tgb:SetAttribute("unit", player);
	tgb:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	tgb:SetScript("OnMouseDown", function() if(arg1 == "RightButton") then rewatch_HidePlayerByName(player); end; end);
	-- add a swiftmend button
	local smb = CreateFrame("BUTTON", nil, rewatch_f, "SecureActionButtonTemplate");
	smb:SetWidth(rewatch_data["SmallButtonWidth"]); smb:SetHeight(rewatch_data["SmallButtonHeight"]); smb:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["swiftmend"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(rewatch_i+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	smb:RegisterForClicks("LeftButtonDown"); smb:SetAttribute("type1", "spell"); smb:SetAttribute("unit", player);
	smb:SetAttribute("spell1", rewatch_loc["swiftmend"]); smb:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	smb:SetNormalTexture("Interface\\Icons\\INV_Relics_IdolofRejuvenation.blp");
	smb:SetScript("OnEnter", function() rewatch_tooltip:ClearLines(); rewatch_tooltip:SetSpell(rewatch_GetSpellId(rewatch_loc["swiftmend"]), BOOKTYPE_SPELL); rewatch_tooltip:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -25, 25); end);
	smb:SetScript("OnLeave", function() rewatch_tooltip:SetOwner(WorldFrame, "ANCHOR_NONE"); end);
	-- create the cooldown overlay
	local smbcd = CreateFrame("Cooldown", nil, smb, "CooldownFrameTemplate");
	smb.cooldown = smbcd; smbcd:SetPoint("CENTER", 0, -1);
	smbcd:SetWidth(rewatch_data["SmallButtonWidth"]); smbcd:SetHeight(rewatch_data["SmallButtonHeight"]); smbcd:Hide();
	-- add abolish poison button
	local apb = CreateFrame("BUTTON", nil, rewatch_f, "SecureActionButtonTemplate");
	apb:SetWidth(rewatch_data["SmallButtonWidth"]); apb:SetHeight(rewatch_data["SmallButtonHeight"]); apb:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["abolishpoison"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(rewatch_i+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	apb:RegisterForClicks("LeftButtonDown"); apb:SetAttribute("type1", "spell"); apb:SetAttribute("unit", player);
	apb:SetAttribute("spell1", rewatch_loc["abolishpoison"]); apb:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	apb:SetNormalTexture("Interface\\Icons\\Spell_Nature_NullifyPoison_02.blp");
	apb:SetScript("OnEnter", function() rewatch_tooltip:ClearLines(); rewatch_tooltip:SetSpell(rewatch_GetSpellId(rewatch_loc["abolishpoison"]), BOOKTYPE_SPELL); rewatch_tooltip:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -25, 25); end);
	apb:SetScript("OnLeave", function() rewatch_tooltip:SetOwner(WorldFrame, "ANCHOR_NONE"); end);
	-- add decurse button
	local dec = CreateFrame("BUTTON", nil, rewatch_f, "SecureActionButtonTemplate");
	dec:SetWidth(rewatch_data["SmallButtonWidth"]); dec:SetHeight(rewatch_data["SmallButtonHeight"]); dec:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["removecurse"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(rewatch_i+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	dec:RegisterForClicks("LeftButtonDown"); dec:SetAttribute("type1", "spell"); dec:SetAttribute("unit", player);
	dec:SetAttribute("spell1", rewatch_loc["removecurse"]); dec:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	dec:SetNormalTexture("Interface\\Icons\\Spell_Shadow_Curse.blp");
	dec:SetScript("OnEnter", function() rewatch_tooltip:ClearLines(); rewatch_tooltip:SetSpell(rewatch_GetSpellId(rewatch_loc["removecurse"]), BOOKTYPE_SPELL); rewatch_tooltip:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -25, 25); end);
	dec:SetScript("OnLeave", function() rewatch_tooltip:SetOwner(WorldFrame, "ANCHOR_NONE"); end);
	-- save player data
	rewatch_bars[rewatch_i]["Player"] = player;
	rewatch_bars[rewatch_i]["PlayerBar"] = statusbar;
	rewatch_bars[rewatch_i]["ManaBar"] = statusbar2;
	rewatch_bars[rewatch_i][rewatch_loc["lifebloom"]] = 0;
	rewatch_bars[rewatch_i]["LifebloomStacks"] = 0;
	rewatch_bars[rewatch_i][rewatch_loc["rejuvenation"]] = 0;
	rewatch_bars[rewatch_i][rewatch_loc["regrowth"]] = 0;
	rewatch_bars[rewatch_i][rewatch_loc["lifebloom"].."Bar"] = rewatch_CreateBar(rewatch_loc["lifebloom"], rewatch_i);
	rewatch_bars[rewatch_i][rewatch_loc["rejuvenation"].."Bar"] = rewatch_CreateBar(rewatch_loc["rejuvenation"], rewatch_i);
	rewatch_bars[rewatch_i][rewatch_loc["regrowth"].."Bar"] = rewatch_CreateBar(rewatch_loc["regrowth"], rewatch_i);
	rewatch_bars[rewatch_i]["SwiftmendButton"] = smb; rewatch_bars[rewatch_i]["AbolishPoisonButton"] = apb; rewatch_bars[rewatch_i]["DecurseButton"] = dec;
	-- increment the global index
	rewatch_i = rewatch_i+1;
	
	-- sort the players table accordingly
	rewatch_SortPlayers();
	
	-- return the inserted player's player table index
	return rewatch_GetPlayer(player);
end;

-- hide all bars and buttons from - and the player himself
-- playerId: the table index of the player to hide
-- return: void
function rewatch_HidePlayer(playerId)
	-- return if in combat
	if(InCombatLockdown() == 1) then return; end;
	
	-- if this action would decrease the number of visible bars, shrink parent frame
	if(rewatch_bars[playerId]["PlayerBar"]:IsVisible()) then
		rewatch_AlterFrame(-1 * (rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	end;
	-- hide the bars
	rewatch_bars[playerId][rewatch_loc["lifebloom"].."Bar"]:Hide();
	rewatch_bars[playerId][rewatch_loc["rejuvenation"].."Bar"]:Hide();
	rewatch_bars[playerId][rewatch_loc["regrowth"].."Bar"]:Hide();
	rewatch_bars[playerId]["PlayerBar"]:Hide();	rewatch_bars[playerId]["SwiftmendButton"]:Hide(); rewatch_bars[playerId]["AbolishPoisonButton"]:Hide();
	rewatch_bars[playerId]["DecurseButton"]:Hide(); rewatch_bars[playerId]["ManaBar"]:Hide();
	
	-- sort the players table accordingly
	rewatch_SortPlayers();
	
	-- return
	return;
end;

-- hide all bars and buttons from - and the player himself, by name
-- player: the name of the player to hide
-- return: void
function rewatch_HidePlayerByName(player)
	-- get the index of this player
	local playerId = rewatch_GetPlayer(player);
	-- if this player exists, hide all bars and buttons from - and the player himself
	if(playerId > 0) then rewatch_HidePlayer(playerId); end;
	
	-- return
	return;
end;

-- create a pressable, textured  button with a cooldown overlay
-- spellName: the name of the spell to create a button for (must be in rewatch_abs and have Offset
-- icon: the name of the icon to be used as texture for the button
-- return: the reference to the button
function rewatch_CreateButton(spellName, icon)
	-- create the button
	local btn = CreateFrame("BUTTON", nil, rewatch_f, "SecureActionButtonTemplate");
	btn:SetWidth(rewatch_data["ButtonWidth"]); btn:SetHeight(rewatch_data["ButtonHeight"]); btn:SetPoint("BOTTOMRIGHT", rewatch_f, "BOTTOMRIGHT", rewatch_abs[spellName]["Offset"], 5);
	btn:RegisterForClicks("LeftButtonDown"); btn:SetAttribute("type1", "spell");
	btn:SetAttribute("spell1", spellName); btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	btn:SetNormalTexture("Interface\\Icons\\"..icon);
	-- create the button overlay
	local btncd = CreateFrame("Cooldown", nil, btn, "CooldownFrameTemplate");
	btn.cooldown = btncd; btncd:SetPoint("CENTER", 0, -1);
	btncd:SetWidth(rewatch_data["ButtonWidth"]); btncd:SetHeight(rewatch_data["ButtonHeight"]); btncd:Hide();
	-- hook tooltip
	btn:SetScript("OnEnter", function() rewatch_tooltip:ClearLines(); rewatch_tooltip:SetSpell(rewatch_GetSpellId(spellName), BOOKTYPE_SPELL); rewatch_tooltip:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", -25, 25); end);
	btn:SetScript("OnLeave", function() rewatch_tooltip:SetOwner(WorldFrame, "ANCHOR_NONE"); end);
	-- add the button reference to the rewatch_abs table
	rewatch_abs[spellName]["Button"] = btn;
	
	-- return the reference to the button
	return btn;
end;

-- create a spell bar with text and add it to the global player table
-- spellName: the name of the spell to create a bar for (must be in rewatch_abs and have Offset)
-- playerId: the index number of the player in the player table
-- return: the created spell bar reference
function rewatch_CreateBar(spellName, playerId)
	-- create the bar
	local b = CreateFrame("STATUSBAR", spellName..playerId, rewatch_f, "TextStatusBar"); b:SetWidth(rewatch_data["SpellBarWidth"]); b:SetHeight(rewatch_data["SpellBarHeight"]); b:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[spellName]["Offset"],  (rewatch_data["SpellBarHeight"]*2)-(playerId+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
	b:SetBackdrop({bgFile = nil, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = true, tileSize = 5, edgeSize = 5, insets = { left = 0, right = 0, top = 0, bottom = 0 }});
	b:SetStatusBarTexture("Interface\\TargetingFrame\\UI-StatusBar"); b:SetStatusBarColor(0.0, 0.0, 0.6, 0.8);
	b:SetMinMaxValues(0, rewatch_abs[spellName]["Duration"]); b:SetValue(0);
	-- put text in status bar
	b.text = b:CreateFontString("$parentText", "ARTWORK", "GameFontHighlightSmall");
	b.text:SetPoint("RIGHT", b); b.text:SetAllPoints(); b.text:SetText(spellName);
	-- overlay cast button
	local bc = CreateFrame("BUTTON", nil, b, "SecureActionButtonTemplate");
	bc:SetWidth(rewatch_data["SpellBarWidth"]); bc:SetHeight(rewatch_data["SpellBarHeight"]); bc:SetPoint("TOPLEFT", b, "TOPLEFT", 0, 0);
	bc:RegisterForClicks("LeftButtonDown"); bc:SetAttribute("type1", "spell"); bc:SetAttribute("unit", rewatch_bars[playerId]["Player"]);
	bc:SetAttribute("spell1", spellName); bc:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square.blp");
	
	-- return the reference to the spell bar
	return b;
end;

-- update a bar by resetting spell duration
-- spellName: the name of the spell to reset it's duration from
-- playerId: the index number of the player in the player table
-- return: void
function rewatch_UpdateBar(spellName, playerId)	
	-- return if playerId (returned from AddPlayer) is invalid
	if(playerId < 0) then return; end;
	
	-- if the player's bar is not visible, make it reappear (or skip appearing when in combat)
	if((not rewatch_bars[playerId]["PlayerBar"]:IsVisible()) and (not (InCombatLockdown() == 1))) then
		-- show all bars
		rewatch_bars[playerId]["PlayerBar"]:Show();
		rewatch_bars[playerId]["ManaBar"]:Show();
		rewatch_bars[playerId][rewatch_loc["lifebloom"].."Bar"]:Show();
		rewatch_bars[playerId][rewatch_loc["rejuvenation"].."Bar"]:Show();
		rewatch_bars[playerId][rewatch_loc["regrowth"].."Bar"]:Show();
		rewatch_bars[playerId]["SwiftmendButton"]:Show();
		rewatch_bars[playerId]["AbolishPoisonButton"]:Show();
		rewatch_bars[playerId]["DecurseButton"]:Show();
		-- refresh the end time and bar value
		rewatch_bars[playerId][spellName] = GetTime() +  rewatch_abs[spellName]["Duration"];
		rewatch_bars[playerId][spellName.."Bar"]:SetValue(rewatch_abs[spellName]["Duration"]);
		-- update lifebloom stack counter
		if(spellName == rewatch_loc["lifebloom"]) then
			rewatch_bars[playerId]["LifebloomStacks"] = math.min(rewatch_bars[playerId]["LifebloomStacks"]+1, 3);
			rewatch_bars[playerId][spellName.."Bar"].text:SetText("Lifebloom x"..rewatch_bars[playerId]["LifebloomStacks"]);
		end;
		-- increase the parent container's height
		rewatch_AlterFrame(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]);
		-- sort the players again
		rewatch_SortPlayers();
	else
		-- refresh the end time and bar value
		rewatch_bars[playerId][spellName] = GetTime() +  rewatch_abs[spellName]["Duration"];
		rewatch_bars[playerId][spellName.."Bar"]:SetValue(rewatch_abs[spellName]["Duration"]);
		-- update lifebloom stack counter
		if(spellName == "Lifebloom") then
			rewatch_bars[playerId]["LifebloomStacks"] = math.min(rewatch_bars[playerId]["LifebloomStacks"]+1, 3);
			rewatch_bars[playerId][spellName.."Bar"].text:SetText("Lifebloom x"..rewatch_bars[playerId]["LifebloomStacks"]);
		end;
	end;
	
	-- return
	return;
end;

-- process a swiftmend cast by the player, on specified playerId
-- playerId: the index number of the player in the player table
-- return: void
function rewatch_ProcessSwiftmend(playerId)
	-- assume it consumed a rejuvenation
	local consumed = rewatch_loc["rejuvenation"];
	-- check if this assumption was wrong, and if so, alter it
	if((rewatch_bars[playerId][consumed] == 0) or (((rewatch_bars[playerId][rewatch_loc["regrowth"]] > 0)) and (rewatch_bars[playerId][rewatch_loc["regrowth"]] < rewatch_bars[playerId][consumed]))) then
		consumed = rewatch_loc["regrowth"];
	end;
	-- clear the bar that was consumed and unset it's end time
	rewatch_bars[playerId][consumed] = 0; rewatch_bars[playerId][consumed.."Bar"]:SetValue(0);
	-- trigger all cooldown overlays of every player's swiftmend button
	for i,val in ipairs(rewatch_bars) do CooldownFrame_SetTimer(val["SwiftmendButton"].cooldown, GetTime(), rewatch_abs["Swiftmend"]["Duration"], 1); end;
	-- also hide an ungrouped player when his hots run out because of a SM
	if((rewatch_bars[playerId][rewatch_loc["rejuvenation"]] == 0) and (rewatch_bars[playerId][rewatch_loc["regrowth"]] == 0) and (rewatch_bars[playerId][rewatch_loc["lifebloom"]] == 0)) then if(not rewatch_InGroup(rewatch_bars[playerId]["Player"])) then rewatch_HidePlayer(playerId); end; end;
	
	-- return
	return;
end;

-- sort the players into a solid lineup (no gaps in between)
-- return: void
function rewatch_SortPlayers()
	-- return if in combat
	if(InCombatLockdown() == 1) then return -1; end;
	
	-- initialise local vars and repeat as long as there's a swap
	local swap, n, m = true, 1, 2; while(swap) do
		-- reset the swap bool
		swap = false;
		-- find a hidden bar which causes a gap
		while(rewatch_bars[n+1]) do
			if(rewatch_bars[n]["PlayerBar"]:IsVisible()) then n = n + 1;
			else break; end;
		end; m = n+1;
		-- find the next visible bar that could fill this gap
		while(rewatch_bars[m]) do
			if(not rewatch_bars[m]["PlayerBar"]:IsVisible()) then m = m + 1;
			else break; end;
		end;
		-- check the found results; they may also be the last+1 index of the table
		if((rewatch_bars[n]) and (rewatch_bars[m])) then
			if((rewatch_bars[m]["PlayerBar"]:IsVisible()) and (not rewatch_bars[n]["PlayerBar"]:IsVisible())) then
				-- swap the found table values
				swap = true; local temp = rewatch_bars[n];
				rewatch_bars[n] = rewatch_bars[m];
				rewatch_bars[n]["PlayerBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, (rewatch_data["SpellBarHeight"]*2)-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n]["ManaBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, rewatch_data["SpellBarHeight"]-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"])+5);
				rewatch_bars[n][rewatch_loc["lifebloom"].."Bar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["lifebloom"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n][rewatch_loc["rejuvenation"].."Bar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["rejuvenation"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n][rewatch_loc["regrowth"].."Bar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["regrowth"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n]["SwiftmendButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["swiftmend"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n]["AbolishPoisonButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["abolishpoison"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[n]["DecurseButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["removecurse"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(n+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m] = temp;
				rewatch_bars[m]["PlayerBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, (rewatch_data["SpellBarHeight"]*2)-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m]["ManaBar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_data["SpellBarMargin"]*2, rewatch_data["SpellBarHeight"]-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"])+5);
				rewatch_bars[m][rewatch_loc["lifebloom"].."Bar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["lifebloom"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m][rewatch_loc["rejuvenation"].."Bar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["rejuvenation"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m][rewatch_loc["regrowth"].."Bar"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["regrowth"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m]["SwiftmendButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["swiftmend"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m]["AbolishPoisonButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["abolishpoison"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
				rewatch_bars[m]["DecurseButton"]:SetPoint("TOPLEFT", rewatch_f, "TOPLEFT", rewatch_abs[rewatch_loc["removecurse"]]["Offset"], (rewatch_data["SpellBarHeight"]*2)-(m+1)*(rewatch_data["SpellBarHeight"] + rewatch_data["SpellBarMargin"]));
			end;
		end;
	end; -- will end when there's no more swap done
	
	-- return
	return;
end;

-- performs a poison check on given unit, will return true if a poison debuff was found
-- playerId: the index number of the player in the player table
-- return: true if the player with the supplied id has a poison, false if not
function rewatch_IsPoisoned(playerId)
	-- go through all possible debuff spots
	local debuffType; for i=1,40 do
		-- get the debuff type, return true if it's poison
		debuffType = select(5, UnitDebuff(rewatch_bars[playerId]["Player"], i, 1));
		if(debuffType == "Poison") then return true;
		-- or, if it's empty, we reached the end of the debuff queue, return false
		elseif((debuffType == "") or (not debuffType)) then return false; end;
	end;
	
	-- return
	return false;
end;

-- performs a curse check on given unit, will return true if a curse debuff was found
-- playerId: the index number of the player in the player table
-- return: true if the player with the supplied id has a curse, false if not
function rewatch_IsCursed(playerId)
	-- go through all possible debuff spots
	local debuffType; for i=1,40 do
		-- get the debuff type, return true if it's a curse
		debuffType = select(5, UnitDebuff(rewatch_bars[playerId]["Player"], i, 1));
		if(debuffType == "Curse") then return true;
		-- or, if it's empty, we reached the end of the debuff queue, return false
		elseif((debuffType == "") or (not debuffType)) then return false; end;
	end;
	
	-- return
	return false;
end;

-- process the sent commands
-- cmd: the command that has to be executed
-- return: void
function rewatch_SlashCommandHandler(cmd)
	-- if the frame is visible
	if(rewatch_f:IsVisible()) then
		-- when there's a command typed
		if(cmd) then
			-- declaration and initialisation
			local pos, commands = 0, {};
			for st, sp in function() return string.find(cmd, " ", pos, true) end do
				table.insert(commands, string.sub(cmd, pos, st-1));
				pos = sp + 1;
			end; table.insert(commands, string.sub(cmd, pos));
			-- on a help request, reply with the localisation help table
			if(commands[1] == "help") then
				for i,val in ipairs(rewatch_loc["help"]) do
					rewatch_Message(val);
				end;
			-- if the user wants to enter the debug mode
			elseif(commands[1] == "debug") then
				-- toggle the debug var
				rewatch_debug = (not rewatch_debug);
				-- report
				rewatch_Message(rewatch_loc["debug"]);
			-- if the user wants to clear the player list
			elseif(commands[1] == "clear") then
				-- hide all bars
				for i,val in ipairs(rewatch_bars) do
					val[rewatch_loc["lifebloom"].."Bar"]:Hide(); val[rewatch_loc["rejuvenation"].."Bar"]:Hide(); val[rewatch_loc["regrowth"].."Bar"]:Hide();
					val["PlayerBar"]:Hide(); val["SwiftmendButton"]:Hide();
					val["AbolishPoisonButton"]:Hide(); val["DecurseButton"]:Hide(); val["ManaBar"]:Hide();
				end;
				-- clear the vars
				rewatch_bars = nil; rewatch_bars = {};
				rewatch_i = 1; rewatch_update = nil;
				-- set the frame height
				rewatch_f:SetHeight(rewatch_data["FrameHeight"]);
				rewatch_f_gcd:SetHeight(rewatch_data["FrameHeight"]);
				-- report
				rewatch_Message(rewatch_loc["cleared"]);
			-- if the user wants something else (unsupported)
			elseif(string.len(commands[1]) > 0) then rewatch_Message(rewatch_loc["invalid_command"]);
			else rewatch_Message(rewatch_loc["credits"]); end;
		-- if there's no command typed
		else rewatch_Message(rewatch_loc["credits"]); end;
	-- else toggle the frame back to visible
	else rewatch_f:Show(); end;
	
	-- return
	return;
end;

-- compares the current player table to the party/raid schedule
-- return: void
function rewatch_ProcessGroup()
	-- return if in combat
	if(InCombatLockdown() == 1) then return; end;
	-- reset changed var
	rewatch_changed = nil;
	-- set group environment
	local env, n;
	if(GetNumRaidMembers() > 0) then env, n = "raid", GetNumRaidMembers();
	elseif(GetNumPartyMembers() > 0) then env, n = "party", GetNumPartyMembers();
	else return; end;
	-- for each group member
	for i=1, n do
		-- if he's not in the list, add him
		local name = UnitName(env..i);
		if(rewatch_GetPlayer(name) == -1) then
			rewatch_AddPlayer(name);
		end;
	end;
	
	-- return
	return;
end;

-- (re)sets the global saved variable table
-- return: void
function rewatch_SetData()
	-- set default scalings
	rewatch_data = {
		FrameWidth = 450; FrameHeight = 50;
		SpellBarWidth = 90; SpellBarHeight = 20; SpellBarMargin = 3;
		SmallButtonWidth = 20; SmallButtonHeight = 20; SmallButtonMargin = 3;
		ButtonWidth = 30; ButtonHeight = 30; ButtonMargin = 3;
	};
	
	-- return
	return;
end;


--// INITIALISE //----------------------------------------------------------------------------------------------------------------------------------------------------------------

-- make the addon stop here if the user isn't a druid
if((select(2, UnitClass("player"))) ~= "DRUID") then return; end;
-- initialise all vars
rewatch_f, rewatch_i, rewatch_bars, rewatch_f_gcd, rewatch_debug = CreateFrame("FRAME", "Rewatch_Frame", UIParent),  1, {}, nil, false;
rewatch_tooltip, rewatch_update, rewatch_changed, btnI, btnNS, btnT, btnBS, btnExit, btnGotW = nil, nil, nil, nil, nil, nil, nil, nil, nil;
-- add the slash command handler
SLASH_REWATCH1 = "/rewatch";
SlashCmdList["REWATCH"] = function(cmd)
	rewatch_SlashCommandHandler(cmd);
end;
-- initialise the data table
rewatch_SetData();
-- initialise the abilities table
rewatch_abs = {
	[rewatch_loc["lifebloom"]] = {
		Duration = 7;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*1;
	};
	[rewatch_loc["rejuvenation"]] = {
		Duration = 12;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*2;
	};
	[rewatch_loc["regrowth"]] = {
		Duration = 21;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*3;
	};
	[rewatch_loc["swiftmend"]] = {
		Duration = 15;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*4 + (rewatch_data["SmallButtonWidth"]+rewatch_data["SmallButtonMargin"])*2;
	};
	[rewatch_loc["abolishpoison"]] = {
		Duration = 1.5;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*4;
	};
	[rewatch_loc["removecurse"]] = {
		Duration = 1.5;
		Offset = rewatch_data["SpellBarMargin"]*2+(rewatch_data["SpellBarWidth"]+rewatch_data["SpellBarMargin"])*4 + (rewatch_data["SmallButtonWidth"]+rewatch_data["SmallButtonMargin"])*1;
	};
	[rewatch_loc["barkskin"]] = {
		Button = btnBS;
		Offset = -5-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*3;
	};
	[rewatch_loc["naturesswiftness"]] = {
		Button = btnNS;
		Offset = -5-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*1;
	};
	[rewatch_loc["innervate"]] = {
		Button = btnI;
		Offset = -5-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*2;
	};
	[rewatch_loc["tranquility"]] = {
		Button = btnT;
		Offset = -5-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*4;
	};
	[rewatch_loc["giftofthewild"]] = {
		Button = btnGotW;
		Offset = -5-(rewatch_data["ButtonWidth"]+rewatch_data["ButtonMargin"])*5;
	};
};


--// SCRIPT //-----------------------------------------------------------------------------------------------------------------------------------------------------------------------

-- create the outline frame
rewatch_f:SetWidth(rewatch_data["FrameWidth"]); rewatch_f:SetHeight(rewatch_data["FrameHeight"]);
rewatch_f:SetPoint("TOPLEFT", UIParent, "TOPLEFT"); rewatch_f:EnableMouse(true); rewatch_f:SetMovable(true);
rewatch_f:SetBackdrop({bgFile = nil, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = 1, tileSize = 10, edgeSize = 10, insets = { left = 3, right = 3, top = 3, bottom = 3 }});
rewatch_f:RegisterEvent("COMBAT_LOG_EVENT_UNFILTERED");
rewatch_f:RegisterEvent("PARTY_MEMBERS_CHANGED");
rewatch_f_gcd = CreateFrame("Cooldown", nil, rewatch_f, "CooldownFrameTemplate");
rewatch_f_gcd:SetPoint("CENTER", 0, -1);
rewatch_f_gcd:SetWidth(rewatch_data["FrameWidth"]); rewatch_f_gcd:SetHeight(rewatch_data["FrameHeight"]); rewatch_f_gcd:Hide();
rewatch_f:SetScript("OnMouseDown", function() rewatch_f:StartMoving(); moving = true; end);
rewatch_f:SetScript("OnMouseUp", function() if(moving) then rewatch_f:StopMovingOrSizing(); moving = false; end; end);
-- make sure this frame catches combat_log_event_unfiltered events and processes them
rewatch_f:SetScript("OnEvent", function(timestamp, event, ...)
	-- if the group/raid the player is in has changed
	if(event == "PARTY_MEMBERS_CHANGED") then		
		-- return if in combat
		if(InCombatLockdown() == 1) then rewatch_changed = true; return; end;
		rewatch_ProcessGroup();
	-- if an aura has been removed from the user
	elseif(arg2 == "SPELL_AURA_REMOVED") then if(arg6 == UnitGUID("player")) then
			-- if the Nature's Swiftness aura has been removed from the user itself, trigger the cooldown overlay
			if(arg10 == rewatch_loc["naturesswiftness"]) then rewatch_update = arg10; end;
		end;
	-- if a spell was cast successfull by the user or a heal has been received
	elseif((arg2 == "SPELL_CAST_SUCCESS") or (arg2 == "SPELL_HEAL")) then if(arg3 == UnitGUID("player")) then
		-- trigger global cooldown
		rewatch_TriggerCooldown();
		-- if this was a regrowth or rejuvenation or LB HoT effect
		if((arg10 == rewatch_loc["regrowth"]) or (arg10 == rewatch_loc["rejuvenation"]) or ((arg10 == rewatch_loc["lifebloom"]) and (arg2 == "SPELL_CAST_SUCCESS"))) then
			-- get the player's table index number
			local playerId = rewatch_GetPlayer(arg7);
			-- if this player doesn't exist in the table yet, add him to it
			if(playerId < 0) then playerId = rewatch_AddPlayer(arg7); end;
			-- update the bar of the corresponding spell and player
			rewatch_UpdateBar(arg10, playerId);
		-- if a swiftmend was received (bugfix: spell launches both events, want to catch only one)
		elseif((arg10 == rewatch_loc["swiftmend"]) and (arg2 == "SPELL_HEAL")) then
			-- get the player's table index number
			local playerId = rewatch_GetPlayer(arg7);
			-- process the cast swiftmend
			rewatch_ProcessSwiftmend(playerId);
		-- if it's a different spell that has a button on the frame, trigger it's cooldown overlay
		elseif((arg10 == rewatch_loc["innervate"]) or (arg10 == rewatch_loc["barkskin"]) or (arg10 == rewatch_loc["tranquility"])) then rewatch_update = arg10; end;
	end; end;
	
	-- return
	return;
end);

-- make a tooltip object
rewatch_tooltip = CreateFrame("GameTooltip", "rewatch_tooltip", WorldFrame, "GameTooltipTemplate");
rewatch_tooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
rewatch_tooltip:AddFontStrings(
    rewatch_tooltip:CreateFontString("$parentTextLeft1", nil, "GameTooltipText"),
	rewatch_tooltip:CreateFontString("$parentTextRight1", nil, "GameTooltipText")
);

-- build exit button
btnExit = CreateFrame("BUTTON", "btnExit", rewatch_f, "OptionsButtonTemplate"); btnExit:SetText("X"); 
btnExit:SetWidth(rewatch_data["ButtonWidth"]); btnExit:SetHeight(rewatch_data["ButtonHeight"]); btnExit:SetPoint("BOTTOMRIGHT", rewatch_f, "BOTTOMRIGHT", -5, 5); 
btnExit:SetScript("OnClick", function() rewatch_f:Hide(); rewatch_Message(rewatch_loc["hidden"]); end);

-- build spell buttons
btnNS = rewatch_CreateButton(rewatch_loc["naturesswiftness"], "Spell_Nature_RavenForm.blp");
btnI = rewatch_CreateButton(rewatch_loc["innervate"], "Spell_Nature_Lightning.blp");
btnBS = rewatch_CreateButton(rewatch_loc["barkskin"], "Spell_Nature_StoneClawTotem.blp");
btnT = rewatch_CreateButton(rewatch_loc["tranquility"], "Spell_Nature_Tranquility.blp");
btnGotW = rewatch_CreateButton(rewatch_loc["giftofthewild"], "Spell_Nature_GiftoftheWild.blp");

-- update everything
rewatch_f:SetScript("OnUpdate", function()	
	-- if the group formation has been changed, add new group members to the list
	if(rewatch_changed) then
		rewatch_ProcessGroup();
	end;
	-- if the update var hasn't been initialised; onload
	if(rewatch_update == nil) then
		-- clear the update var
		rewatch_update = "";
		-- add the user itself to the list by default
		rewatch_AddPlayer(UnitName("player"));
		-- process the group initially
		rewatch_ProcessGroup();
		-- trigger the cooldown overlays
		for s,val in pairs(rewatch_abs) do rewatch_TriggerCooldown(s); end;
	-- else, if it's not empty, trigger a cooldown
	elseif(rewatch_update ~= "") then
		-- trigger the cooldown overlay
		rewatch_TriggerCooldown(rewatch_update);
		-- clear the update var
		rewatch_update = "";
	-- else, for each set of player data
	else
		for i,val in ipairs(rewatch_bars) do
			-- update player health bars if possible
			local a, b = val["PlayerBar"]:GetMinMaxValues(); if(true or (b > 0)) then
				-- get and set mana data
				a, b = UnitManaMax(val["Player"]), UnitMana(val["Player"]);
				val["ManaBar"]:SetMinMaxValues(0, a); val["ManaBar"]:SetValue(b);
				-- get and set health data
				a, b = UnitHealthMax(val["Player"]), UnitHealth(val["Player"]);
				val["PlayerBar"]:SetMinMaxValues(0, a); val["PlayerBar"]:SetValue(b);
				-- clear buffs if the player died (set it to 1 so it will be seen as expired)
				if((b <= 0) and (a > 0)) then val[rewatch_loc["rejuvenation"]], val[rewatch_loc["regrowth"]], val[rewatch_loc["lifebloom"]] = 1, 1, 1;
				else
					-- set healthbar color accordingly
					if(b/a < .25) then val["PlayerBar"]:SetStatusBarColor(0.6, 0.0, 0.0, 0.9);
					elseif(a == b) then val["PlayerBar"]:SetStatusBarColor(0.0, 0.8, 0.0, 0.9);
					else val["PlayerBar"]:SetStatusBarColor(0.0, 0.6, 0.0, 0.8); end;
					-- check for poison and set the button accordingly
					if(rewatch_IsPoisoned(i)) then val["AbolishPoisonButton"]:SetNormalTexture("Interface\\Icons\\Spell_Nature_NullifyPoison_02.blp");
					else val["AbolishPoisonButton"]:SetNormalTexture(nil); end;
					-- check for poison and set the button accordingly
					if(rewatch_IsCursed(i)) then val["DecurseButton"]:SetNormalTexture("Interface\\Icons\\Spell_Shadow_Curse.blp");
					else val["DecurseButton"]:SetNormalTexture(nil); end;
					-- set manabar accordingly power type (energy, rage, mana)
					a = UnitPowerType(val["Player"]);
					if(a == 0) then val["ManaBar"]:SetStatusBarColor(0.0, 0.0, 0.6, 0.8);
					elseif(a == 1) then val["ManaBar"]:SetStatusBarColor(0.6, 0.0, 0.0, 0.8);
					else val["ManaBar"]:SetStatusBarColor(0.95, 0.85, 0.15, 0.8); end;
				end;
			end;
			-- if no rejuvenation was cast on this player
			if(val[rewatch_loc["rejuvenation"]] == 0) then -- nuffin!
			-- else, if it has just expired
			elseif(GetTime() >= val[rewatch_loc["rejuvenation"]]) then
				-- clear the end time and bar value
				val[rewatch_loc["rejuvenation"].."Bar"]:SetValue(0); val[rewatch_loc["rejuvenation"]] = 0;
				-- if this player is not in the group and has no other HoTs on him, remove him from the pane (hide)
				-- if((val[rewatch_loc["regrowth"]] == 0) and (val[rewatch_loc["lifebloom"]] == 0)) then if(not rewatch_InGroup(val["Player"])) then rewatch_HidePlayer(i); end; end;
			-- else update the bar's value
			else
				local left = val[rewatch_loc["rejuvenation"]]-GetTime();
				val[rewatch_loc["rejuvenation"].."Bar"]:SetValue(left);
				if(left > 2) then val[rewatch_loc["rejuvenation"].."Bar"]:SetStatusBarColor(0.0, 0.0, 0.6, 0.8);
				else val[rewatch_loc["rejuvenation"].."Bar"]:SetStatusBarColor(0.6, 0.0, 0.0, 0.8); end;
			end;
			-- if no regrowth was cast on this player
			if(val[rewatch_loc["regrowth"]] == 0) then --nuffin!
			-- else, if it has just expired
			elseif(GetTime() >= val[rewatch_loc["regrowth"]]) then
				-- clear the end time and bar value
				val[rewatch_loc["regrowth"].."Bar"]:SetValue(0); val[rewatch_loc["regrowth"]] = 0;
				-- if this player is not in the group and has no other HoTs on him, remove him from the pane (hide)
				-- if((val[rewatch_loc["rejuvenation"]] == 0) and (val[rewatch_loc["lifebloom"]] == 0)) then if(not rewatch_InGroup(val["Player"])) then rewatch_HidePlayer(i); end; end;
			-- else update the bar's value
			else
				local left = val[rewatch_loc["regrowth"]]-GetTime();
				val[rewatch_loc["regrowth"].."Bar"]:SetValue(left);
				if(left > 2) then val[rewatch_loc["regrowth"].."Bar"]:SetStatusBarColor(0.0, 0.0, 0.6, 0.8);
				else val[rewatch_loc["regrowth"].."Bar"]:SetStatusBarColor(0.6, 0.0, 0.0, 0.8); end;
			end;
			-- if no regrowth was cast on this player
			if(val[rewatch_loc["lifebloom"]] == 0) then --nuffin!
			-- else, if it has just expired
			elseif(GetTime() >= val[rewatch_loc["lifebloom"]]) then
				-- clear the end time and bar value
				val[rewatch_loc["lifebloom"].."Bar"]:SetValue(0); val[rewatch_loc["lifebloom"]] = 0;
				val["LifebloomStacks"] = 0; val[rewatch_loc["lifebloom"].."Bar"].text:SetText(rewatch_loc["lifebloom"]);
				-- if this player is not in the group and has no other HoTs on him, remove him from the pane (hide)
				-- if((val[rewatch_loc["rejuvenation"]] == 0) and (val[rewatch_loc["regrowth"]] == 0)) then if(not rewatch_InGroup(val["Player"])) then rewatch_HidePlayer(i); end; end;
			-- else update the bar's value
			else
				local left = val[rewatch_loc["lifebloom"]]-GetTime();
				val[rewatch_loc["lifebloom"].."Bar"]:SetValue(left);
				if(left > 2) then val[rewatch_loc["lifebloom"].."Bar"]:SetStatusBarColor(0.0, 0.0, 0.6, 0.8);
				else val[rewatch_loc["lifebloom"].."Bar"]:SetStatusBarColor(0.6, 0.0, 0.0, 0.8); end;
			end;
			-- if this player is not in the group and has no other HoTs on him, remove him from the pane (hide)
			if((val[rewatch_loc["rejuvenation"]] == 0) and (val[rewatch_loc["lifebloom"]] == 0) and (val[rewatch_loc["regrowth"]] == 0)) then if(not rewatch_InGroup(val["Player"])) then rewatch_HidePlayer(i); end; end;
		end;
	end;
	
	-- return
	return;
end);
